@@ -1,11 +1,12 @@
 package HTTP::Server::EV::Buffer;
-our $VERSION = '0.6';
+our $VERSION = '0.65';
 
 use strict;
 use bytes;
 use EV;
 use Scalar::Util qw/weaken/;
 use Carp;
+use IO::Handle;
 
 =head1 NAME
 
@@ -45,11 +46,11 @@ sub new {
 	my ($name, $self) = @_;
 	
 	$self->{flush} = $autoflush unless exists $self->{flush};
+
 	
-	unless ($self->{fh}){
-		open $self->{fh}, '>&='.$self->{fd}; 
-		binmode $self->{fh};
-	}
+	$self->{fh} ||= IO::Handle->new_from_fd($self->{fd}, 'r+' )
+		or croak 'Cant create filehandle from fd!';
+	$self->{fh}->blocking ( 0 );
 	
 	# break circular because onneror possible contains closure with H:S:E::CGI object that contains ref to buffer
 	# weaken $self->{onerror} if $self->{onerror};
@@ -87,13 +88,19 @@ sub new {
 				if( length $self->{buffer} ){
 					return;
 				}else{
-					if( $self->{status} == 1 ){
+					if($self->{flush_cb}){
+						$self->{flush_cb}->();
+						$self->{flush_cb} = undef;
+					}
+					
+					if( $self->{status} == 1 ){ # status 1 - just flush data
 						$self->{status} = 0;
 					}else{ # staus = 2 (all flushed and close) or -1 (client closed socket)
 						delete $buffers{$self->{fd}};
 						$_[0]->stop;
 						return; # now it calls DESTROY (or not calls if status -1 and other code holds reference) again and closes the socket
 					}
+					
 				}
 			}
 			$_[0]->stop;
@@ -105,6 +112,7 @@ sub new {
 }
 
 
+
 sub TIEHANDLE { # pkgname, buffer obj
 	return $_[1];
 }
@@ -120,9 +128,6 @@ Prints data to buffer.
 
 
 
-
-
-
 sub WRITE {
 	return if $_[0]->{status} < 0;
 	$_[0]->{buffer} .=  substr $_[1], 0, $_[2];
@@ -160,13 +165,21 @@ sub onerror {$_[0]->{onerror} = $_[1] }
 
 Sends all buffered data to socket and sets new flush threshold if $flush_threshold defined;
 
+=head2 $buffer->flush_wait( sub{} )
+
+->flush only starts sending data in async background mode. ->flush_wait starts flushing and calls callback when all data is sent. If callback isn't specified it will try use Coro to block current thread (threading in PortListener must be enabled). If threading disabled and no callback specified it works as ->flush
+
 =head2 $buffer->{onerror} = sub {} or  $buffer->onerror(sub {})
 
 Set onerror callback, which called when client disconeted before cerver closed connection. 
 
+=head2 $buffer->give_up_handle
+
+Use when you need do some bi-directional communication through client socket(for ex. websocket server or make CONNECT proxy). Returns filehandle opened for RW and now buffer won't do anything with handle - $buffer will be set to undef; Call ->flush_wait before calling ->give_up_handle if you have data in buffer, otherwise it will be lost
+
 =head2 $buffer->{error_w}->start
 
-Call this after ->new if you use H:S:E:Buffer without HTTP::Server::EV and need onerror callback. Don't think about it when use buffer object from HTTP::Server::EV::CGI object. 
+Call this after ->new if you use H:S:E:Buffer without HTTP::Server::EV and need onerror callback. It's called automatically after receiving GET or POST request, for other methods you must start watcher manually after reading all data from handle.
 
 =cut
 
@@ -181,10 +194,33 @@ sub flush {
 		$_[0]->{w}->start;
 	}
 }
+sub flush_wait {
+	if(length $_[0]->{buffer}){
+		$_[0]->{status} = 1;
+		$_[0]->{w}->start;
+		
+		if($_[1]){
+			$_[0]->{flush_cb} = $_[1];
+		} else{
+			$_[0]->{flush_cb} = Coro::rouse_cb();
+			return Coro::rouse_wait();
+		}
+	}
+}
 
+sub give_up_handle {
+	$_[0]->{_deleted} = 1; # disable destructor
+	
+	my $fh = $_[0]->{fh};
+	
+	$_[0] = undef;
+	
+	$fh;
+}
 
 
 sub DESTROY {
+	return if $_[0]->{_deleted};
 	if(length $_[0]->{buffer}){
 		$_[0]->{status} = 2;
 		$_[0]->{w}->start;
